package com.webonlineapp;
	
import com.hyjavaapex.chart.Apexchart;
import com.hyjavacharts.chart.Highchart;
import com.hyjavastock.chart.Highstock;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import javafx.stage.Stage;

/*
 * A simple proof of concept example application to show how to integrate
 *   Highcharts and the HyJavaCharts library
 * plus
 *   Highcharts Stock and the HyJavaStock library
 * plus
 *   ApexCharts and the HyJavaApex library
 *   
 * in a JavaFX application.
 * 
 * NOTE: We are not experts in JavaFx so there will no doubt be better ways to accomplish the same thing.
 * 
 */
public class Main extends Application {

    @Override
    public void start(Stage primaryStage) {
    	primaryStage.setTitle("HyJavaCharts, HyJavaStock and HyJavaApex JavaFX Demo");
    	VBox root = new VBox();
    	
        WebView webView = new WebView();
        WebEngine engine = webView.getEngine();

        HBox buttons = new HBox();
        buttons.setSpacing(10);
        buttons.setPadding(new Insets(15, 12, 15, 12));
        
        Button lineChartButton = new Button("Highcharts Line Chart");
        buttons.getChildren().add(lineChartButton);
        
        Button pieChartButton = new Button("Highcharts Pie Chart");
        buttons.getChildren().add(pieChartButton);
        
        Button stockChartButton = new Button("Highstock Column Chart");
        buttons.getChildren().add(stockChartButton);
        
        Button apexChartButton = new Button("ApexCharts Column Chart");
        buttons.getChildren().add(apexChartButton);
        
        root.getChildren().add(buttons);
        root.getChildren().add(webView);
        
        lineChartButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override public void handle(ActionEvent e) {
            	engine.loadContent(buildContent("line"));
            }
        });
        
        pieChartButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override public void handle(ActionEvent e) {
            	engine.loadContent(buildContent("pie"));
            }
        });
        
        stockChartButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override public void handle(ActionEvent e) {
            	engine.loadContent(buildContent("stock"));
            }
        });
        
        apexChartButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override public void handle(ActionEvent e) {
            	engine.loadContent(buildContent("apex"));
            }
        });

        primaryStage.setScene(new Scene(new BorderPane(root)));
        primaryStage.show();
    }
    
    /**
     * 
     * @return
     */
    private String buildContent(String chart) {
    	String javascript = "<script src=https://code.highcharts.com/stock/8.2.2/highstock.js></script>\r\n" + 
			    			"<script src=https://code.highcharts.com/stock/8.2.2/modules/data.js></script>\r\n" + 
			    			"<script src=\"https://cdn.jsdelivr.net/npm/apexcharts@3.23.1\"></script>\\r\\n" + 
			        		"<div id=\"container\" style=\"min-width: 310px; max-width: 800px; height: 400px; margin: 0 auto\"></div>\r\n" + 
			        		"<script>\r\n";
    	
    	if (chart.equals("stock")) {
    		Highstock hs = new HighstockColumn().configure();
    		
    		javascript = javascript + 
    			(hs.globalOptionsToJs() == null ? "" : "Highcharts.setOptions(" + hs.globalOptionsToJs() + ");\r\n") + 
        		"Highcharts.stockChart('container', " + hs.chartOptionsToJs() + " );\r\n";
    	}
    	else if (chart.equals("apex")) {
    		Apexchart apexchart = new ApexChartsBasicColumn().configure();
    		
    		javascript = javascript +
    			"var chart = new ApexCharts(document.querySelector('#container'), " + apexchart.chartOptionsToJs() + ");\r\n" + 
    			"chart.render();\r\n";
    	}
    	else {
    		Highchart hc = null;
        	
        	if (chart.equals("line")) {
        		hc = new HighchartsLine().configure();
        	}
        	else {
        		hc = new HighchartsPie().configure();
        	}
        	
        	javascript = javascript + 
    			(hc.globalOptionsToJs() == null ? "" : "Highcharts.setOptions(" + hc.globalOptionsToJs() + ");\r\n") + 
        		"Highcharts.chart('container', " + hc.chartOptionsToJs() + " );\r\n";
    	}
    	
    	javascript = javascript + "</script>";

    	return javascript;
    }
    
    /**
     * 
     * @param args
     */
	public static void main(String[] args) {
		launch(args);
	}
}
