package com.webonlineapp;

import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import com.hyjavaapex.chart.Apexchart;
import com.hyjavacharts.chart.Highchart;
import com.hyjavastock.chart.Highstock;
import com.webonlineapp.charts.ApexChartsBasicColumn;
import com.webonlineapp.charts.HighchartsLine;
import com.webonlineapp.charts.HighchartsPie;
import com.webonlineapp.charts.HighstockColumn;

import javafx.application.Platform;
import javafx.embed.swing.JFXPanel;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;

/*
 * A simple proof of concept example application to show how to integrate
 *   Highcharts and the HyJavaCharts library
 * plus
 *   Highcharts Stock and the HyJavaStock library
 * plus
 *   ApexCharts and the HyJavaApex library
 *   
 * in a Java Swing application.
 * 
 * NOTE: We are not experts in Swing/JavaFX so there will no doubt be better ways to accomplish the same thing.
 * 
 */
public class DemoUI {
	private WebView webView;
	private WebEngine engine;

	/**
	 * 
	 */
	public DemoUI() {
		JFrame frame = new JFrame();
		frame.setTitle("HyJavaCharts, HyJavaStock and HyJavaApex Swing Demo");
		frame.setSize(new Dimension(850, 650));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLocationRelativeTo(null);
		frame.setResizable(false);
		frame.setVisible(true);

		JPanel main = new JPanel();
		BoxLayout boxlayout = new BoxLayout(main, BoxLayout.Y_AXIS);
		main.setLayout(boxlayout);

		JPanel buttons = new JPanel(new FlowLayout(FlowLayout.LEFT, 8, 8));
		JButton lineChartButton = new JButton("Highcharts Line Chart");
		buttons.add(lineChartButton);

		JButton pieChartButton = new JButton("Highcharts Pie Chart");
		buttons.add(pieChartButton);

		JButton stockChartButton = new JButton("Highstock Column Chart");
		buttons.add(stockChartButton);
		
		JButton apexChartButton = new JButton("ApexCharts Column Chart");
		buttons.add(apexChartButton);

		main.add(buttons);

		JFXPanel fxPanel = new JFXPanel();
		main.add(fxPanel);

		frame.add(main);

		// line chart button
		lineChartButton.addActionListener((e) -> {
			Platform.runLater(new Runnable() {
				@Override
				public void run() {
					engine.loadContent(buildContent("line"));
				}
			});
		});

		// pie chart button
		pieChartButton.addActionListener((e) -> {
			Platform.runLater(new Runnable() {
				@Override
				public void run() {
					engine.loadContent(buildContent("pie"));
				}
			});
		});

		// stock chart button
		stockChartButton.addActionListener((e) -> {
			Platform.runLater(new Runnable() {
				@Override
				public void run() {
					engine.loadContent(buildContent("stock"));
				}
			});
		});
		
		// apex chart button
		apexChartButton.addActionListener((e) -> {
			Platform.runLater(new Runnable() {
				@Override
				public void run() {
					engine.loadContent(buildContent("apex"));
				}
			});
		});

		Platform.runLater(new Runnable() {
			@Override
			public void run() {
				webView = new WebView();
				engine = webView.getEngine();

				initFX(fxPanel);
			}
		});
	}

	/**
	 * 
	 * @param fxPanel
	 */
	private void initFX(JFXPanel fxPanel) {
		// This method is invoked on the JavaFX thread
		Scene scene = createScene();
		fxPanel.setScene(scene);
	}

	/**
	 * 
	 * @return
	 */
	private Scene createScene() {
		Group root = new Group();
		Scene scene = new Scene(root);

		root.getChildren().add(webView);

		return scene;
	}

	/**
	 * 
	 * @param chart
	 * @return
	 */
	private String buildContent(String chart) {
		String javascript = "<script src=https://code.highcharts.com/stock/8.2.2/highstock.js></script>\r\n"
				+ "<script src=https://code.highcharts.com/stock/8.2.2/modules/data.js></script>\r\n"
				+ "<script src=\"https://cdn.jsdelivr.net/npm/apexcharts@3.23.1\"></script>\\r\\n"
				+ "<div id=\"container\" style=\"min-width: 310px; max-width: 800px; height: 400px; margin: 0 auto\"></div>\r\n"
				+ "<script>\r\n";

		if (chart.equals("stock")) {
			Highstock hs = new HighstockColumn().configure();

			javascript = javascript
					+ (hs.globalOptionsToJs() == null ? ""
							: "Highcharts.setOptions(" + hs.globalOptionsToJs() + ");\r\n")
					+ "Highcharts.stockChart('container', " + hs.chartOptionsToJs() + " );\r\n";
		} 
		else if (chart.equals("apex")) {
			Apexchart apexchart = new ApexChartsBasicColumn().configure();
    		
    		javascript = javascript
    			+ "var chart = new ApexCharts(document.querySelector('#container'), " + apexchart.chartOptionsToJs() + ");\r\n" 
    			+ "chart.render();\r\n";
			
		}
		else {
			Highchart hc = null;

			if (chart.equals("line")) {
				hc = new HighchartsLine().configure();
			} else {
				hc = new HighchartsPie().configure();
			}

			javascript = javascript
					+ (hc.globalOptionsToJs() == null ? ""
							: "Highcharts.setOptions(" + hc.globalOptionsToJs() + ");\r\n")
					+ "Highcharts.chart('container', " + hc.chartOptionsToJs() + " );\r\n";
		}

		javascript = javascript + "</script>";

		return javascript;
	}

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}

		SwingUtilities.invokeLater(() -> {
			new DemoUI();
		});
	}

}
